﻿using Android.App;
using Android.Content;
using Android.Content.PM;
using Android.Graphics;
using Android.OS;
using Android.Views;
using Android.Widget;
using AndroidX.AppCompat.Content.Res;
using AndroidX.AppCompat.Widget;
using AT.Markushi.UI;
using Newtonsoft.Json;
using QuickDate.Activities.Base;
using QuickDate.Helpers.CacheLoaders;
using QuickDate.Helpers.Controller;
using QuickDate.Helpers.Utils;
using QuickDateClient.Requests;
using System;
using System.Linq;
using Console = System.Console;
using Toolbar = AndroidX.AppCompat.Widget.Toolbar;

namespace QuickDate.PaymentUtil
{
    [Activity(Icon = "@mipmap/icon", Theme = "@style/MyTheme", ConfigurationChanges = ConfigChanges.Locale | ConfigChanges.UiMode | ConfigChanges.ScreenSize | ConfigChanges.Orientation | ConfigChanges.ScreenLayout | ConfigChanges.SmallestScreenSize)]
    public class PaymentLocalActivity : BaseActivity
    {
        #region Variables Basic

        private TextView CardNumber, CardCode, CardCountry, CardName, TxtNote;
        private ImageView Image;
        private CircleButton ImageClose;
        private AppCompatButton BtnAddImage, BtnApply;
        private string Price, PayType, Credits, Id, PathImage = "";

        #endregion

        #region General

        protected override void OnCreate(Bundle savedInstanceState)
        {
            try
            {
                base.OnCreate(savedInstanceState);

                Methods.App.FullScreenApp(this);
                SetTheme(QuickDateTools.IsTabDark() ? _Microsoft.Android.Resource.Designer.ResourceConstant.Style.MyTheme_Dark : _Microsoft.Android.Resource.Designer.ResourceConstant.Style.MyTheme);

                // Create your application here
                SetContentView(_Microsoft.Android.Resource.Designer.ResourceConstant.Layout.PaymentLocalLayout);

                Id = Intent?.GetStringExtra("Id") ?? "";
                Credits = Intent?.GetStringExtra("credits") ?? "";
                Price = Intent?.GetStringExtra("Price");
                PayType = Intent?.GetStringExtra("payType"); // credits|membership 

                //Get Value And Set Toolbar
                InitComponent();
                InitBackground();
                InitToolbar();
            }
            catch (Exception e)
            {
                Methods.DisplayReportResultTrack(e);
            }
        }

        protected override void OnResume()
        {
            try
            {
                base.OnResume();
                AddOrRemoveEvent(true);
            }
            catch (Exception e)
            {
                Methods.DisplayReportResultTrack(e);
            }
        }

        protected override void OnPause()
        {
            try
            {
                base.OnPause();
                AddOrRemoveEvent(false);
            }
            catch (Exception e)
            {
                Methods.DisplayReportResultTrack(e);
            }
        }

        public override void OnTrimMemory(TrimMemory level)
        {
            try
            {
                GC.Collect(GC.MaxGeneration, GCCollectionMode.Forced);
                base.OnTrimMemory(level);
            }
            catch (Exception e)
            {
                Methods.DisplayReportResultTrack(e);
            }
        }

        public override void OnLowMemory()
        {
            try
            {
                GC.Collect(GC.MaxGeneration);
                base.OnLowMemory();
            }
            catch (Exception e)
            {
                Methods.DisplayReportResultTrack(e);
            }
        }

        #endregion

        #region Menu

        public override bool OnOptionsItemSelected(IMenuItem item)
        {
            switch (item.ItemId)
            {
                case Android.Resource.Id.Home:
                    Finish();
                    return true;
            }
            return base.OnOptionsItemSelected(item);
        }

        #endregion

        #region Functions

        private void InitComponent()
        {
            try
            {
                CardNumber = (TextView)FindViewById(_Microsoft.Android.Resource.Designer.ResourceConstant.Id.card_number);
                CardCode = (TextView)FindViewById(_Microsoft.Android.Resource.Designer.ResourceConstant.Id.card_code);
                CardCountry = (TextView)FindViewById(_Microsoft.Android.Resource.Designer.ResourceConstant.Id.card_country);
                CardName = (TextView)FindViewById(_Microsoft.Android.Resource.Designer.ResourceConstant.Id.card_name);
                TxtNote = (TextView)FindViewById(_Microsoft.Android.Resource.Designer.ResourceConstant.Id.note);
                Image = (ImageView)FindViewById(_Microsoft.Android.Resource.Designer.ResourceConstant.Id.Image);

                ImageClose = (CircleButton)FindViewById(_Microsoft.Android.Resource.Designer.ResourceConstant.Id.ImageCircle);
                BtnAddImage = (AppCompatButton)FindViewById(_Microsoft.Android.Resource.Designer.ResourceConstant.Id.btn_AddPhoto);
                BtnApply = (AppCompatButton)FindViewById(_Microsoft.Android.Resource.Designer.ResourceConstant.Id.ApplyButton);

                var currencySymbol = ListUtils.SettingsSiteList?.CurrencySymbol ?? "$";
                if (PayType == "credits")
                {
                    //Please transfer the amount of $100 to this bank account to purchase "Box of Credits 5000"

                    TxtNote.Text = GetString(_Microsoft.Android.Resource.Designer.ResourceConstant.String.Lbl_short_Note1) + " " + currencySymbol + Price + " " +
                                   GetString(_Microsoft.Android.Resource.Designer.ResourceConstant.String.Lbl_short_Note2) + " " + Credits;
                }
                else
                {
                    //Please transfer the amount of $500 to this bank account to purchase "Lifetime" Plan Premium Membership

                    TxtNote.Text = GetString(_Microsoft.Android.Resource.Designer.ResourceConstant.String.Lbl_short_Note1) + " " + currencySymbol + Price + " " +
                                   GetString(_Microsoft.Android.Resource.Designer.ResourceConstant.String.Lbl_short_Note2) + " " + Credits + " " +
                                   GetString(_Microsoft.Android.Resource.Designer.ResourceConstant.String.Lbl_short_Note3);
                }

                var splitText = ListUtils.SettingsSiteList?.BankDescription.Split(new[] { "<p>", "</p>" }, StringSplitOptions.None);
                Console.WriteLine(splitText);

                if (splitText != null)
                {
                    CardNumber.Text = splitText[1];
                    CardName.Text = splitText[3];
                    CardCode.Text = splitText[5];
                    CardCountry.Text = splitText[7];
                }
            }
            catch (Exception e)
            {
                Methods.DisplayReportResultTrack(e);
            }
        }

        private void InitToolbar()
        {
            try
            {
                var toolbar = FindViewById<Toolbar>(_Microsoft.Android.Resource.Designer.ResourceConstant.Id.toolbar);
                if (toolbar != null)
                {
                    toolbar.Title = GetString(_Microsoft.Android.Resource.Designer.ResourceConstant.String.Lbl_BankTransfer);
                    toolbar.SetTitleTextColor(QuickDateTools.IsTabDark() ? AppSettings.TitleTextColorDark : AppSettings.TitleTextColor);
                    SetSupportActionBar(toolbar);
                    SupportActionBar.SetDisplayShowCustomEnabled(true);
                    SupportActionBar.SetDisplayHomeAsUpEnabled(true);
                    SupportActionBar.SetHomeButtonEnabled(true);
                    SupportActionBar.SetDisplayShowHomeEnabled(true);

                    var icon = AppCompatResources.GetDrawable(this, AppSettings.FlowDirectionRightToLeft ? _Microsoft.Android.Resource.Designer.ResourceConstant.Drawable.icon_back_arrow_right : _Microsoft.Android.Resource.Designer.ResourceConstant.Drawable.icon_back_arrow_left);
                    icon?.SetTint(QuickDateTools.IsTabDark() ? Color.White : Color.Black);
                    SupportActionBar.SetHomeAsUpIndicator(icon);
                }
            }
            catch (Exception e)
            {
                Methods.DisplayReportResultTrack(e);
            }
        }

        private void AddOrRemoveEvent(bool addEvent)
        {
            try
            {
                // true +=  // false -=
                if (addEvent)
                {
                    ImageClose.Click += ImageCloseOnClick;
                    BtnAddImage.Click += BtnAddImageOnClick;
                    BtnApply.Click += BtnApplyOnClick;
                }
                else
                {
                    ImageClose.Click -= ImageCloseOnClick;
                    BtnAddImage.Click -= BtnAddImageOnClick;
                    BtnApply.Click -= BtnApplyOnClick;
                }
            }
            catch (Exception e)
            {
                Methods.DisplayReportResultTrack(e);
            }
        }

        #endregion

        #region Events

        private void BtnAddImageOnClick(object sender, EventArgs e)
        {
            try
            {
                PixImagePickerUtils.OpenDialogGallery(this);
            }
            catch (Exception exception)
            {
                Methods.DisplayReportResultTrack(exception);
            }
        }

        private async void BtnApplyOnClick(object sender, EventArgs e)
        {
            try
            {
                if (Methods.CheckConnectivity())
                {
                    if (string.IsNullOrEmpty(PathImage) || string.IsNullOrWhiteSpace(PathImage))
                    {
                        Toast.MakeText(this, GetText(_Microsoft.Android.Resource.Designer.ResourceConstant.String.Lbl_ErrorPleaseSelectImage), ToastLength.Long)?.Show();
                        return;
                    }

                    var (apiStatus, respond) = await RequestsAsync.Common.UploadBankRecipeAsync(PayType, Price, "Pay the card " + Price, PathImage);
                    if (apiStatus == 200)
                    {
                        Toast.MakeText(this, GetText(_Microsoft.Android.Resource.Designer.ResourceConstant.String.Lbl_YourWasReceiptSuccessfullyUploaded), ToastLength.Long)?.Show();
                    }
                    else Methods.DisplayReportResult(this, respond);
                }
                else
                {
                    Toast.MakeText(this, GetText(_Microsoft.Android.Resource.Designer.ResourceConstant.String.Lbl_CheckYourInternetConnection), ToastLength.Long)?.Show();
                }
            }
            catch (Exception exception)
            {
                Methods.DisplayReportResultTrack(exception);
            }
        }

        private void ImageCloseOnClick(object sender, EventArgs e)
        {
            try
            {
                PathImage = "";
                GlideImageLoader.LoadImage(this, "Grey_Offline", Image, ImageStyle.CenterCrop, ImagePlaceholders.Drawable);
            }
            catch (Exception exception)
            {
                Methods.DisplayReportResultTrack(exception);
            }
        }

        #endregion

        #region Permissions && Result

        //Result
        protected override void OnActivityResult(int requestCode, Result resultCode, Intent data)
        {
            try
            {
                base.OnActivityResult(requestCode, resultCode, data);

                if (requestCode == PixImagePickerActivity.RequestCode && resultCode == Result.Ok)
                {
                    var listPath = JsonConvert.DeserializeObject<ResultIntentPixImage>(data.GetStringExtra("ResultPixImage") ?? "");
                    if (listPath?.List?.Count > 0)
                    {
                        var filepath = listPath.List.FirstOrDefault();
                        if (!string.IsNullOrEmpty(filepath))
                        {
                            //Do something with your Uri
                            PathImage = filepath;
                            GlideImageLoader.LoadImage(this, filepath, Image, ImageStyle.CenterCrop, ImagePlaceholders.Drawable);
                        }
                    }
                }
            }
            catch (Exception e)
            {
                Methods.DisplayReportResultTrack(e);
            }
        }

        //Permissions
        public override void OnRequestPermissionsResult(int requestCode, string[] permissions, Permission[] grantResults)
        {
            try
            {
                base.OnRequestPermissionsResult(requestCode, permissions, grantResults);

                if (requestCode == 106)
                {
                    if (grantResults.Length > 0 && grantResults[0] == Permission.Granted)
                    {
                        PixImagePickerUtils.OpenDialogGallery(this);
                    }
                    else
                    {
                        Toast.MakeText(this, GetText(_Microsoft.Android.Resource.Designer.ResourceConstant.String.Lbl_Permission_is_denied), ToastLength.Long)?.Show();
                    }
                }
            }
            catch (Exception e)
            {
                Methods.DisplayReportResultTrack(e);
            }
        }

        #endregion

    }
}